var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { Components } from 'formiojs';
var FieldComponent = Components.components.field;
import editForm from './CascadeSelect.form';
import myPicker from 'my-picker';
import NativePromise from 'native-promise-only';
import _ from 'lodash';
var CascadeSelect = /** @class */ (function (_super) {
    __extends(CascadeSelect, _super);
    function CascadeSelect(component, options, data) {
        var _this = _super.call(this, component, options, data) || this;
        _this.checks = [];
        return _this;
    }
    CascadeSelect.schema = function () {
        return FieldComponent.schema({
            type: 'cascadeselect',
            label: 'Cascade Select',
            key: 'cascadeSelect',
            data: {
                json: '',
                url: '',
                headers: []
            },
            dataSrc: 'json',
            placeholder: 'Please Select',
            labelProperty: 'label',
            valueProperty: 'value',
            childrenProperty: 'children',
            numSelects: 2
        });
    };
    CascadeSelect.prototype.init = function () {
        var _this = this;
        _super.prototype.init.call(this);
        this.validators = this.validators.concat(['select']);
        // Trigger an update.
        var updateArgs = [];
        var triggerUpdate = _.debounce(function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            updateArgs = [];
            return _this.updateItems.apply(_this, args);
        }, 100);
        this.triggerUpdate = function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            if (args.length) {
                updateArgs = args;
            }
            return triggerUpdate.apply(void 0, updateArgs);
        };
        // Keep track of the select options.
        this.selectOptions = [];
        if (this.isInfiniteScrollProvided) {
            this.isFromSearch = false;
            this.searchServerCount = null;
            this.defaultServerCount = null;
            this.isScrollLoading = false;
            this.searchDownloadedResources = [];
            this.defaultDownloadedResources = [];
        }
        // If this component has been activated.
        this.activated = false;
        // Determine when the items have been loaded.
        this.itemsLoaded = new NativePromise(function (resolve) {
            _this.itemsLoadedResolve = resolve;
        });
    };
    Object.defineProperty(CascadeSelect.prototype, "dataReady", {
        get: function () {
            return this.itemsLoaded;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CascadeSelect.prototype, "emptyValue", {
        get: function () {
            return [];
        },
        enumerable: true,
        configurable: true
    });
    /* eslint-disable max-statements */
    CascadeSelect.prototype.setItems = function (items) {
        // If the items is a string, then parse as JSON.
        if (typeof items == 'string') {
            try {
                items = JSON.parse(items);
            }
            catch (err) {
                console.warn(err.message);
                items = [];
            }
        }
        // Allow js processing (needed for form builder)
        if (this.component.onSetItems && typeof this.component.onSetItems === 'function') {
            var newItems = this.component.onSetItems(this, items);
            if (newItems) {
                items = newItems;
            }
        }
        // If they provided select values, then we need to get them instead.
        if (this.component.selectValues) {
            items = _.get(items, this.component.selectValues, items) || [];
        }
        var options = items;
        for (var i = 0; i < this.component.numSelects; i++) {
            this.picker.setOptions(i, options);
            options = (options[0] && options[0][this.component.childrenProperty]) || [];
        }
        if (!(this.selectedOptions && this.selectedOptions.length) && (this.defaultValue && this.defaultValue.length)) {
            this.setValue(this.defaultValue);
        }
        // Say we are done loading the items.
        this.itemsLoadedResolve();
    };
    /* eslint-enable max-statements */
    CascadeSelect.prototype.loadItems = function (url, headers, options, method, body) {
        var _this = this;
        options = options || {};
        // Ensure we have a method and remove any body if method is get
        method = method || 'GET';
        if (method.toUpperCase() === 'GET') {
            body = null;
        }
        var query = {
            sort: null
        };
        // Allow for url interpolation.
        url = this.interpolate(url, {
            formioBase: Formio.getBaseUrl(),
        });
        // Add sort capability
        if (this.component.sort) {
            query.sort = this.component.sort;
        }
        if (!_.isEmpty(query)) {
            // Add the query string.
            url += (!url.includes('?') ? '?' : '&') + Formio.serialize(query, function (item) { return _this.interpolate(item); });
        }
        // Add filter capability
        if (this.component.filter) {
            url += (!url.includes('?') ? '?' : '&') + this.interpolate(this.component.filter);
        }
        // Make the request.
        options.header = headers;
        this.loading = true;
        Formio.makeRequest(this.options.formio, 'select', url, method, body, options)
            .then(function (response) {
            _this.loading = false;
            _this.setItems(response);
        })
            .catch(function (err) {
            if (_this.isInfiniteScrollProvided) {
                _this.setItems([]);
            }
            _this.isScrollLoading = false;
            _this.loading = false;
            _this.itemsLoadedResolve();
            _this.emit('componentError', {
                component: _this.component,
                message: err.toString(),
            });
            console.warn("Unable to load resources for " + _this.key);
        });
    };
    Object.defineProperty(CascadeSelect.prototype, "requestHeaders", {
        /**
       * Get the request headers for this select dropdown.
       */
        get: function () {
            var _this = this;
            // Create the headers object.
            var headers = new Formio.Headers();
            // Add custom headers to the url.
            if (this.component.data && this.component.data.headers) {
                try {
                    _.each(this.component.data.headers, function (header) {
                        if (header.key) {
                            headers.set(header.key, _this.interpolate(header.value));
                        }
                    });
                }
                catch (err) {
                    console.warn(err.message);
                }
            }
            return headers;
        },
        enumerable: true,
        configurable: true
    });
    /* eslint-disable max-statements */
    CascadeSelect.prototype.updateItems = function () {
        var _this = this;
        this.itemsLoaded = new NativePromise(function (resolve) {
            _this.itemsLoadedResolve = resolve;
        });
        if (!this.component.data) {
            console.warn("Select component " + this.key + " does not have data configuration.");
            this.itemsLoadedResolve();
            return;
        }
        // Only load the data if it is visible.
        if (!this.checkConditions()) {
            this.itemsLoadedResolve();
            return;
        }
        switch (this.component.dataSrc) {
            case 'json':
                this.setItems(this.component.data.json);
                break;
            case 'url': {
                var url = this.component.data.url;
                var method = void 0;
                var body = void 0;
                if (url.startsWith('/')) {
                    // if URL starts with '/project', we should use base URL to avoid issues with URL formed like <base_url>/<project_name>/project/<project_id>/...
                    var baseUrl = url.startsWith('/project') ? Formio.getBaseUrl() : Formio.getProjectUrl() || Formio.getBaseUrl();
                    url = baseUrl + url;
                }
                if (!this.component.data.method) {
                    method = 'GET';
                }
                else {
                    method = this.component.data.method;
                    if (method.toUpperCase() === 'POST') {
                        body = this.component.data.body;
                    }
                    else {
                        body = null;
                    }
                }
                this.loadItems(url, this.requestHeaders, { noToken: true }, method, body);
                break;
            }
        }
    };
    /* eslint-enable max-statements */
    /**
   * Activate this select control.
   */
    CascadeSelect.prototype.activate = function () {
        if (this.active) {
            return;
        }
        this.activated = true;
        this.triggerUpdate();
    };
    Object.defineProperty(CascadeSelect.prototype, "active", {
        get: function () {
            return !this.component.lazyLoad || this.activated || this.options.readOnly;
        },
        enumerable: true,
        configurable: true
    });
    CascadeSelect.prototype.updateSelectedLabels = function () {
        if (!this.selectedOptions) {
            return;
        }
        var selectedLabels = '';
        for (var _i = 0, _a = this.selectedOptions; _i < _a.length; _i++) {
            var selectedOption = _a[_i];
            if (selectedOption && selectedOption[this.component.labelProperty]) {
                selectedLabels += (selectedOption[this.component.labelProperty] + ' ');
            }
        }
        if (!selectedLabels) {
            selectedLabels = this.component.placeholder;
        }
        this.refs.selectedLabels.innerText = selectedLabels;
    };
    CascadeSelect.prototype.render = function () {
        return _super.prototype.render.call(this, this.renderTemplate('cascadeselect'));
    };
    CascadeSelect.prototype.attach = function (element) {
        var _this = this;
        this.loadRefs(element, {
            selectHandle: 'single',
            selectedLabels: 'single'
        });
        var selectHandle = this.refs.selectHandle;
        var component = this.component;
        var cols = [];
        for (var i = 0; i < component.numSelects; i++) {
            cols.push({
                options: [],
                labelKey: component.labelProperty,
                valueKey: component.valueProperty,
            });
        }
        if (this.picker) {
            this.picker.close();
            this.picker = null;
            this.selectedOptions = null;
        }
        this.picker = myPicker({
            cols: cols,
            title: component.label,
            zIndex: 10000,
            onSelectItem: function (level, selectedIndex, selectedValue) {
                var nextLevelOptions = this.getOptions(level);
                var index = selectedIndex;
                for (var i = level + 1; i < component.numSelects; i++) {
                    nextLevelOptions = (nextLevelOptions[index] && nextLevelOptions[index][component.childrenProperty]) || [];
                    index = 0;
                    this.setOptions(i, nextLevelOptions);
                }
            },
            onOkClick: function (values) {
                _this.updateSelectedOptions();
                _this.updateValue();
                return true;
            }
        });
        selectHandle.addEventListener('click', function () {
            _this.picker.show();
        });
        this.triggerUpdate();
        return _super.prototype.attach.call(this, element);
    };
    CascadeSelect.prototype.updateSelectedOptions = function () {
        var _this = this;
        this.selectedOptions = null;
        var _loop_1 = function (i) {
            var options = this_1.picker.getOptions(i);
            if (options && options.length) {
                var selectedValue_1 = this_1.picker.getValue(i);
                var selectedOption = options && selectedValue_1 && options.find(function (option) { return option[_this.component.valueProperty] === selectedValue_1; });
                if (selectedOption) {
                    selectedOption = _.cloneDeep(selectedOption);
                    delete selectedOption.children;
                }
                this_1.selectedOptions = this_1.selectedOptions || [];
                this_1.selectedOptions[i] = selectedOption;
            }
        };
        var this_1 = this;
        for (var i = 0; i < this.component.numSelects; i++) {
            _loop_1(i);
        }
        this.updateSelectedLabels();
    };
    /**
     * Get the value of the component from the dom elements.
     *
     * @returns {Array}
     */
    CascadeSelect.prototype.getValue = function () {
        return this.selectedOptions;
    };
    /**
     * Set the value of the component into the dom elements.
     *
     * @param value
     * @returns {boolean}
     */
    CascadeSelect.prototype.setValue = function (value) {
        if (!value || !value.length || !this.picker) {
            return;
        }
        for (var i = 0; i < this.component.numSelects; i++) {
            var selectedValue = value[i] && value[i][this.component.valueProperty];
            this.picker.setValue(i, selectedValue);
        }
        this.updateSelectedOptions();
    };
    CascadeSelect.prototype.detach = function () {
        _super.prototype.detach.call(this);
        if (this.picker) {
            this.picker.close();
            this.picker = null;
            this.selectedOptions = null;
        }
    };
    CascadeSelect.editForm = editForm;
    CascadeSelect.builderInfo = {
        title: 'Cascade Select',
        group: 'basic',
        icon: 'th-list',
        weight: 75,
        documentation: 'http://help.form.io/userguide/#select',
        schema: CascadeSelect.schema()
    };
    return CascadeSelect;
}(FieldComponent));
export default CascadeSelect;
