export default [
    {
        key: 'multiple',
        ignore: true
    },
    {
        type: 'number',
        input: true,
        key: 'numSelects',
        weight: 0,
        label: 'Number Of Select Dropdowns',
        defaultValue: 2,
        tooltip: 'Number Of Select Dropdowns.'
    },
    {
        type: 'select',
        input: true,
        weight: 1,
        tooltip: 'The source to use for the select data. Values lets you provide your own values and labels. JSON lets you provide raw JSON data. URL lets you provide a URL to retrieve the JSON data from.',
        key: 'dataSrc',
        defaultValue: 'values',
        label: 'Data Source Type',
        dataSrc: 'values',
        data: {
            values: [
                { label: 'URL', value: 'url' },
                { label: 'Raw JSON', value: 'json' }
            ],
        },
    },
    {
        type: 'textarea',
        as: 'json',
        editor: 'ace',
        weight: 10,
        input: true,
        key: 'data.json',
        label: 'Data Source Raw JSON',
        tooltip: 'A raw JSON array to use as a data source.',
        conditional: {
            json: { '===': [{ var: 'data.dataSrc' }, 'json'] },
        },
    },
    {
        type: 'textfield',
        input: true,
        key: 'data.url',
        weight: 11,
        label: 'Data Source URL',
        placeholder: 'Data Source URL',
        tooltip: 'A URL that returns a JSON array to use as the data source.',
        conditional: {
            json: { '===': [{ var: 'data.dataSrc' }, 'url'] },
        },
    },
    {
        type: 'datagrid',
        input: true,
        label: 'Request Headers',
        key: 'data.headers',
        tooltip: 'Set any headers that should be sent along with the request to the url. This is useful for authentication.',
        weight: 12,
        components: [
            {
                label: 'Key',
                key: 'key',
                input: true,
                type: 'textfield',
            },
            {
                label: 'Value',
                key: 'value',
                input: true,
                type: 'textfield',
            },
        ],
        conditional: {
            json: { '===': [{ var: 'data.dataSrc' }, 'url'] },
        },
    },
    {
        type: 'textfield',
        input: true,
        label: 'Data Path',
        key: 'selectValues',
        weight: 13,
        description: 'The object path to the iterable items.',
        tooltip: 'The property within the source data, where iterable items reside. For example: results.items or results[0].items',
        conditional: {
            json: { '===': [{ var: 'data.dataSrc' }, 'url'] },
        },
    },
    {
        type: 'textfield',
        input: true,
        label: 'Label Property',
        key: 'labelProperty',
        skipMerge: true,
        clearOnHide: false,
        weight: 15,
        description: "The property that is used for the option label.",
        tooltip: 'The property of each item in the data source to use as the option label.',
        validate: {
            required: true
        },
        conditional: {
            json: {
                in: [
                    { var: 'data.dataSrc' },
                    [
                        'json',
                        'url'
                    ],
                ],
            },
        },
    },
    {
        type: 'textfield',
        input: true,
        label: 'Value Property',
        key: 'valueProperty',
        skipMerge: true,
        clearOnHide: false,
        weight: 16,
        description: "The selected item's property to save.",
        tooltip: 'The property of each item in the data source to use as the select value. If not specified',
        validate: {
            required: true
        },
        conditional: {
            json: {
                in: [
                    { var: 'data.dataSrc' },
                    [
                        'json',
                        'url'
                    ],
                ],
            },
        },
    },
    {
        type: 'textfield',
        input: true,
        label: 'Children Property',
        key: 'childrenProperty',
        skipMerge: true,
        clearOnHide: false,
        weight: 17,
        description: "The selected item's property to populate the options for the next select box.",
        tooltip: 'The property of each item in the data source to use as the options of the next select box.',
        validate: {
            required: true
        },
        conditional: {
            json: {
                in: [
                    { var: 'data.dataSrc' },
                    [
                        'json',
                        'url'
                    ],
                ],
            },
        },
    },
    {
        type: 'textfield',
        input: true,
        key: 'filter',
        label: 'Filter Query',
        weight: 18,
        description: 'The filter query for results.',
        tooltip: 'Use this to provide additional filtering using query parameters.',
        conditional: {
            json: {
                in: [
                    { var: 'data.dataSrc' },
                    [
                        'url'
                    ],
                ],
            },
        },
    },
    {
        type: 'textfield',
        input: true,
        key: 'sort',
        label: 'Sort Query',
        weight: 19,
        description: 'The sort query for results',
        tooltip: 'Use this to provide additional sorting using query parameters',
        conditional: {
            json: {
                in: [
                    { var: 'data.dataSrc' },
                    [
                        'url'
                    ],
                ],
            },
        },
    }
];
